/*
 * BustaGraphicsSprites.h
 * Sprites Class. Loads all the sprites in the game and provides them to BustaGraphics
 *
 * Authors: Julian Chan, Thomas Patecky
 * Created: July 15, 2005
 * Last Modified: 5:41pm, July 25, 2005 
 *
 */
#ifndef BUSTAGRAPHICSSPRITES_H
#define BUSTAGRAPHICSSPRITES_H

#include "../../SDL/include/SDL.h"
#include "Font.h"
#include "../models/BustaModelGameState.h"

//defines what color in the sprite should be considered transparent
#define TRANSPARENT_KEY_R			0
#define TRANSPARENT_KEY_G			255
#define TRANSPARENT_KEY_B			0

//All sprites goes under this folder.
#define PATH_SPRITES				"./Resource/sprites"
#define	MAX_FILE_LENGTH				256

//Food sprites
//** Look inside LoadFoodPieces() for the list of food pieces
#define PATH_FOOD_PIECE_SPRITES		"FoodPieces"
#define FOOD_PIECE_SIZE				50	
enum FoodFreshness { FOOD_FRESH, FOOD_STALE, FOOD_ROTTEN, FOOD_NUM_FRESH_STATES};


//Main Screen
#define PATH_MAIN_SCREEN			"MainScreen"
#define BMP_MAIN_SCREEN_LOGO		"MainScrnTitle.bmp"

//Chef animation properties
#define NUM_CHEFS					1
#define ANIM_CHEF_NUM_FRAMES		5
#define CHEF_HEIGHT					250
#define CHEF_WIDTH					250
#define PATH_CHEF_SPRITES			"Chefs"
#define ANIM_CHEF_DELAY				5	//How many frames to delay before rendering next sprite
enum ANIM_CHEF_TYPES { CHEF_HEALTHY, CHEF_BLOATED, CHEF_DYING, CHEF_NUM_STATES };

//Customer animation properties
#define NUM_CUSTOMERS				1
#define CUSTOMER_NUM_FRAMES			4
#define ANIM_CUSTOMER_DELAY			5
#define PATH_CUSTOMER_SPRITES		"Patrons"
#define CUSTOMER_HEIGHT				100
#define CUSTOMER_WIDTH				100	
enum ANIM_CUSTOMER_STATES { CUSTOMER_CONTENT, CUSTOMER_PISSED, CUSTOMER_NUM_STATES };


//BMP Font
#define BMP_FONT					"stdFont.bmp"

//Cursor properties
#define PATH_CURSOR					"Cursor"
#define PATH_SWAP_CURSOR			"SwapCursor"

//Tip Jar
#define PATH_TIP_JAR				"TipJar"
#define TIP_JAR_STAGES				4

/**
 * @class BustaGraphicsSprites
 * @brief contains methods for loading and returning sprites for BustaGraphics
 *
 */
class BustaGraphicsSprites
{
		private:			
				
			//Sprites Storage
			SDL_Surface* food_sprites[NUMFOODPIECES][FOOD_NUM_FRESH_STATES];
			SDL_Surface* font;
			SDL_Surface* cursor_sprites;
			SDL_Surface* swap_cursor_sprites;
			SDL_Surface* chef_sprites[NUM_CHEFS][CHEF_NUM_STATES][ANIM_CHEF_NUM_FRAMES];
			SDL_Surface* main_screen_logo;
			SDL_Surface* tip_jar_sprites[TIP_JAR_STAGES];
			SDL_Surface* customer_sprites[NUM_CUSTOMERS][CUSTOMER_NUM_STATES][CUSTOMER_NUM_FRAMES];
			
			/**
			 * @brief loads all the food sprites in the order specified by the enum FoodType
			 */
			void LoadFoodPieces();
			
			/**
			 * @brief loads a bitmap font
			 */
			void LoadFonts();
			
			/**
			 * @brief loads the chef sprites
			 */
			void LoadChefSprites();
			
			/**
			 * @brief loads the sprites for the cursor and swap cursor
			 */
			void LoadCursorSprites();
	
			/**
			 * @brief loads the logo for main screen
			 */
			void LoadMainScreenLogo();
			
			/**
			 * @brief loads the tip jar sprites
			 */
			void LoadTipJarSprites();
			
			/**
			 * @brief loads the tip jar sprites
			 */
			void LoadCustomerSprites();
			
			/**
			 * @brief cuts out the transparent part of a sprite
			 */
			void ApplyTransparency(SDL_Surface *surface);

		public:
			/**
			 * @brief Loads all the sprites in the game
			 */
			BustaGraphicsSprites();
			
			SDL_Surface* getMainScreenLogo() { return main_screen_logo;}
			
			/**
			 * @brief Returns the propper sprite for a food piece
			 * @param food_piece the food piece to get sprite for
			 * @return sprite for a food type
			 */	
			SDL_Surface* getFoodSprite(BustaModelFoodPiece* food_piece);
			
			/**
			 * @brief Returns a bitmap font used by Sfont
			 * @return a surface for SFont
			 */
			SDL_Surface* getBMPFont() { return font; }
			
			/**
			 * @brief Returns the right chef sprite
			 * @param anim_type the type of animation
			 * @param frame which frame of the animation
			 * @return sprite for the chef
			 */
			SDL_Surface* getChefSprites(ANIM_CHEF_TYPES anim_type, int frame);
			
			/**
			 * @brief Returns the right chef sprite
			 * @param anim_type the type of animation
			 * @param frame which frame of the animation
			 * @return sprite for the chef
			 */
			SDL_Surface* getCustomerSprites(ANIM_CUSTOMER_STATES anim_type, int frame);
			
			/**
			 * @brief returns a sprite for the cursor
			 * @return sprite for the cursor
			 */
			SDL_Surface* getCursorSprites() {return cursor_sprites; }
			
			/**
			 * @brief returns a sprite for the swap cursor
			 * @return sprite for the swap cursor
			 */
			SDL_Surface* getSwapCursorSprites() { return swap_cursor_sprites; }

			/**
			 * @brief returns a sprite of the tip jar based on the percent fullness
			 * @param percent_full 100 for a full jar and 50 for a half-full jar
			 * @return sprite for tip jar
			 */
			SDL_Surface* getTipJarSprite(int percent_full);
};


#endif
