/*	Filename:		BustaNetClient.h
 *	Description:	Busta' Sandwich client module.  Interfaces with game engine and server
 *
 */

#ifndef BUSTA_SANDWICH_CLIENT
#define BUSTA_SANDWICH_CLIENT

#include "../../SDL/include/SDL_net.h"
#include "../../BustaSandwichd/BustaNetMessages.h"
#include "BustaNetInternal.h"
#include "BustaNetMsgQ.h"
#include "../models/BustaModelGameState.h"
#include "../models/BustaModelMPGameState.h"
#include "../models/BustaModelCharacter.h"
#include "../models/BustaModelSpecialSandwich.h"
#include <string.h>

/**
 * @class BustaNetClient
 * @brief Client network interface to Busta'Sandwich
 *
 * Busta'Sandwich Client for networked multiplayer games.
 */
class BustaNetClient
{
	public:
		/**
		 * @brief Constructor
		 *
		 * Initializes variables
		 */
		BustaNetClient();

		/**
		 * @brief Destructor
		 *
		 * Close any open connections and shutdown SDLNet
		 */
		~BustaNetClient();

		/**
		 * @brief Initializes the client and prepares it for connection
		 *
		 * Initializes SDLNet.  Relays any error message back.
		 *
		 * @return Returns a cstring containing an error message if SDLNet cannot be initialized.
		 *	On no error, null is returned.
		 */
		char* init();

		/**
		 * @brief Connects to a server using a given port
		 *
		 * The client initializes a TCP connection with the server on the specified port.
		 *
		 * @param IP The IP address of the server to connect to.
		 * @param serverPort The port of the server to connect to.
		 *
		 * @return A cstring containing information about the connection.  Returns NULL on successful
		 *	connection to the server, or a cstring about connection failure on an unsuccessful connection.
		 */
		char* connect(char* IP, Uint16 serverPort);

		/**
		 * @brief Disconnects from the server
		 *
		 * The connection between the server and client is disconnected.  Subsequent connections may
		 *	be made using the client
		 */
		void disconnect();

		/**
		 * @brief Returns game state information about opponent players
		 *
		 * Returns a pointer to a BustaModelMPGameState containing information about opponent players.
		 *
		 * @return A BustaModelMPGameState containing opponent game states
		 *
		 * @see BustaModelMPGameState
		 */
		BustaModelMPGameState* getMPGS();

		/**
		 * @brief Returns the server priority message queue
		 *
		 * Returns the queue of messages from the server.  These messages do not cover special sandwiches or
		 *	game states.
		 *
		 * @return A BustaNetMsgQ pointer to a server message queue
		 *
		 * @see BustaNetMsgQ
		 */
		BustaNetMsgQ* getPriorityQ();

		/**
		 * @brief Returns the special sandwich queue
		 *
		 * Returns the queue of special sandwiches sent between players in a multiplayer game.
		 *
		 * @return A BustaNetMsgQ pointer to a special sandwich message queue
		 *
		 * @see BustaNetMsgQ
		 */
		BustaNetMsgQ* getSandwichQ();

		/**
		 * @brief Returns the name of the server connected to
		 *
		 * Returns the name of the server the client is currently connected to.
		 *
		 * @return The name of the server
		 */
		char* getServerName();

		/**
		 * @brief Joins the server multiplayer game
		 *
		 * This method attempts to connect the client to the multiplayer game hosted by the server.  The
		 *	client should be connected to the server before running this method.
		 *
		 * @param playerName The name the player wishes to use for the multiplayer game.  Names longer than
		 *	20 characters will be truncated.
		 *
		 * @return A MPConnectResponse with the server's response to the client's connect attempt.  Returns 
		 *	null if the client is not already connected to the server.
		 *
		 * @see connect()
		 */
		MPConnectResponse* joinGame(char* playerName);

		/**
		 * @brief Handles all incoming network data
		 *
		 * Processes all data currently queued on the TCP socket.  The data is partitioned out into
		 *	three message containers.  Each of these three message containers is accessible through
		 *	an accompanying accessor method.
		 *
		 * @see getMPGS()
		 * @see getPriorityQ()
		 * @see getSandwichQ()
		 */
		void processNet();

		/**
		 * @brief Sends character selection of player
		 *
		 * Sends the character selection by the player to the server to be sent to all connected
		 *	players.
		 *
		 * @param character The selected character
		 */
		void sendCharSelect(BustaModelCharacter character);

		/**
		 * @brief Sends current character selection cursor position of player
		 *
		 * Sends the current position of the player's cursor during multiplayer character selection.
		 *
		 * @param cursor The cursor position of the player's character selection cursor
		 */
		void sendCursor(BustaModelPosition* cursor);

		/**
		 * @brief Notifies the server that the player has lost the game
		 *
		 * A message is sent to the server to notify it that the player has lost in the current
		 *	multiplayer game.
		 */
		void sendGameOver();

		/**
		 * @brief Tells the server which opponent the player wishes to view
		 *
		 * A message is sent to the server to tell it which opponent the player wishes to have a
		 *	detailed view of.  The detailed view contains information to display the entire
		 *	screen of the opponent rather than limited brief data.
		 *
		 * @param opponentID The identification of the opponent whom the player wishes to view
		 */
		void sendOpponentViewReq(int opponentID);

		/**
		 * @brief Sends a ready message to the server
		 *
		 * Sending a ready message to the server indicates that the client is ready to begin the
		 *	multiplayer game.  The ready state is visible to all other clients and is used to determine
		 *	when the server should transition from the accepting connections state.
		 */
		void sendReady();

		/**
		 * @brief Sends a spcial sandwich across the network
		 *
		 * A special sandwich is sent from the player to an opposing player.
		 *
		 * @param opponentID The identification of the opponent to whom the player wishes to send the sandwich
		 * @param sandwich The special sandwich to send
		 */
		void sendSandwich(int opponentID, SpecialSandwich sandwich);

		/**
		 * @brief Sends player's game state to the server
		 *
		 * A BustaModelGameState is converted into structs without pointers to form a packet
		 *  to send to the server.  It makes two sends, a packet identifier message, and the
		 *  netGameState data packet.
		 *
		 * @param gs The game state of the player
		 */
		void sendState(BustaModelGameState* gs);

	private:
		IPaddress serverIP;					// IP address of the game server
		char* serverName;					// server name
		BustaModelMPGameState* MPGameState;	// Opponent players' game states
		BustaNetMsgQ* PriorityQ;			// Queue of server messages
		BustaNetMsgQ* SandwichQ;			// Queue of special sandwich sends

		SDLNet_SocketSet socketset;			// Socketset containing server connection socket
		TCPsocket tcpsock;					// Socket connection to server
};

#endif
